local _G = _G
local date = _G.date
local MAJOR_VERSION = 'GraveLevelBar'

local LibStub = _G.LibStub
local AceAddon = LibStub('AceAddon-3.0')
local addon = AceAddon:GetAddon(MAJOR_VERSION)

local min = _G.min
local max = _G.max

addon.repBarTemplate = {}
local repBarTemplate = addon.repBarTemplate

-- start xp
function repBarTemplate:getStartXPMin()
    return -42000
end

-- target level
function repBarTemplate:getTargetLevelMax()
    local currentLevel = self:getCurrentLevel()
    local dataMax =  #REP_DATA
    return max(currentLevel,dataMax)
end

function repBarTemplate:getTargetXPMax()
    local dataMax = self:calcXpForLevel(#REP_DATA)
    local currentXP = self:getCurrentXP()
    return max(dataMax,currentXP)
end

function repBarTemplate:calcXpForLevel(level)
    --assert(level <= 8,"level=" .. tostring(level))
    if level > 8 then
        return self:calcXpForLevel(8)
    end
    local sum = repBarTemplate:getStartXPMin()
    for i=1,level-1 do
        sum = sum + REP_DATA[i]
    end
    return sum
end

function repBarTemplate:calcLevelForXP(xp)
    local level = 1
    while xp > self:calcXpForLevel(level+1) and level <= 8 do
        level = level + 1
    end
    local startXP = self:calcXpForLevel(level)
    assert(startXP <= xp,"level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    if startXP == xp then
        return level
    end
    local nextXP = self:calcXpForLevel(level+1)
    local fraction
    if nextXP > startXP then
        fraction = (xp-startXP)/(nextXP-startXP)
    else
        fraction = 1
    end
    assert(0.0 <= fraction and fraction <= 1.0, "fraction=" .. fraction .. ", level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    return level + fraction
end


function addon:createRepModule(name,...)
    assert(addon.barTemplate.optionsTable.type)
    local template = addon:merge(addon.barTemplate, addon.xpBarTemplate)
    local template2 = addon:merge(template, repBarTemplate)
    assert(template2.optionsTable.type)
    template2.optionsTable.args.startXP.name = "Start Rep"
    template2.optionsTable.args.targetXP.name = "Target Rep"
    local module = addon:NewModule(name,template2,'AceHook-3.0',...)
    return module
end

